<?php
namespace BK\Route;

use BK\Record\WorkorderLine;
use BK\Route;
use BK\Utils;
use http\Exception;

class Setting extends Route {
    public $template = 'settings';
    public $public = false;

    /**
     * @var \Monolog\Logger
     */
    public $logger;

    public function _get() {
        $setting = new \BK\Record\Setting($this->app['db']);
        $backups = [];
        foreach (array_diff(scandir($this->app->_dir->base . '\src'), array('..', '.')) as $file) {
            if (stripos($file, 'bookkeeper') !== false) {
                $backups[] = $file;
            }
        }
        $this->upgrade();
        $this->assign('themes', array_diff(scandir($this->app->_dir->base . '\public\css\theme'), array('..', '.')));
        $this->assign('businessHeader', $setting->get('business-header'));
        $this->assign('databases', $backups);
    }

    public function _post_theme() {
        $setting = new \BK\Record\Setting($this->app['db']);
        $setting->set('theme', $this->params['theme']);
    }

    public function _post_update() {
        file_put_contents($this->app->_dir->base . '\src\session.json', json_encode([]));
        $this->assign('session', true);
    }

    public function _post_set() {
        if (!isset($this->params['field']) || !isset($this->params['value']) || (isset($this->params['field']) && trim($this->params['field']) == '')) {
            throw new \Exception('Unable to set setting');
        }
        $setting = new \BK\Record\Setting($this->app['db']);
        $setting->set($this->params['field'], $this->params['value']);
    }

    public function _post_databaseDelete($id) {
        if (file_exists($this->app->_dir->base . '\src\\' . $id)) {
            unlink($this->app->_dir->base . '\src\\' . $id);
        }

    }

    public function _post_databaseBackup($id) {
        copy($this->app->_dir->base . '\src\\' . $id, $this->app->_dir->base . '\src\\' . $this->params['name']);
    }

    public function _post_databaseRestore($id) {
        copy($this->app->_dir->base . '\src\bookkeeper.db',
            $this->app->_dir->base . '\src\bookkeeper_retired_' . date('Y_m_d') . '.db');

        copy($this->app->_dir->base . '\src\\' . $id,
            $this->app->_dir->base . '\src\bookkeeper.db');



    }

    public function _post_databasePreview($id) {
        $_SESSION['_db']['working'] = $id;
    }

    public function _post_install() {
        $installPath = str_replace('\www','',$this->app->_dir->base);
        $installFile = $installPath . '\Update.zip';
        file_put_contents($installFile, file_get_contents('https://bookkeeper.appstash.in/Update.zip'));
        $zip = new \ZipArchive();
        $res = $zip->open($installFile);
        if ($res === TRUE) {
            $zip->extractTo($installPath . '\\www\\');
            $zip->close();
            Utils::removeDirectory($installPath . '\www\__MACOSX');
            unlink($installFile);
            $this->assign('install', true);
        } else {
            $this->assign('install', false);
        }
    }

    public function upgrade() {
        if (!isset($_SESSION['_upgrade']['1.0.4'])) {
            $workorderLine = new WorkorderLine($this->app['db']);
            if (!isset($workorderLine->columns['workorder_line']['work_notes'])) {
                $workorderLine->db->query("ALTER TABLE workorder_line RENAME COLUMN `name` TO `work_notes`");
            }
            if (!isset($workorderLine->columns['workorder_line']['item_list'])) {
                $workorderLine->db->query("ALTER TABLE workorder_line ADD COLUMN `item_list` TEXT");
            }
            $_SESSION['_upgrade']['1.0.4'] = true;
        }
    }
}
