<?php
namespace BK\Route;

use BK\Record\Entity;
use BK\Record\InvoiceLine;
use BK\Record\Vehicle;
use BK\Record\WorkorderLine;
use BK\Route;

class Workorder extends Route {
    public $template = 'workorder';
    public $public = false;

    /**
     * @var \Monolog\Logger
     */
    public $logger;
    public $record = 'workorder';
    public $short = 'WO';

    public function _get() {
        $this->assign('_type', $this->record);
        $this->assign('_short', $this->short);
    }

    public function _get_list() {
        $this->setTemplate($this->record . '/list');
        $workorder = new \BK\Record\Workorder($this->app['db']);
        $transaction = new \BK\Record\Transaction($this->app['db']);
        $transaction->audit();
        $this->assign('list', $workorder->listAll());
    }

    public function _get_editor($id) {
        $attachments = new \BK\Record\Attachment($this->app['db']);
        $transaction = new \BK\Record\Transaction($this->app['db']);
        $customer = new \BK\Record\Customer($this->app['db']);
        $vehicle = new Vehicle($this->app['db']);
        $workorder = new \BK\Record\Workorder($this->app['db']);
        $workorderLine = new WorkorderLine($this->app['db']);
        $invoice = new \BK\Record\Invoice($this->app['db']);
        $invoiceLine = new InvoiceLine($this->app['db']);
        $data = $workorder->load($id);
        $data['lines'] = $workorderLine->getByWorkOrder($id);
        if (empty($data['invoice_id'])) {
            $inv = $invoice->getByWorkOrder($id);
            $data['invoice_id'] = (!empty($inv) ? $inv['id'] : null);
        }
        $this->setTemplate($this->record . '/editor');
        $this->assign('data', $data);
        $this->assign('totals', [
            'workorder_cost' => $workorderLine->getTotalDueByWorkOrder($id),
            'workorder_paid' => $workorder->getTotalPaidByWorkOrder($id),
            'invoice_cost' => $invoiceLine->getTotalByWorkOrder($id)
        ]);
        $this->assign('payments', $transaction->getPaymentsByWorkOrder($id));
        $this->assign('attachments', $attachments->listAll());

        $this->assign('categories', \BK\Utils::workOrderCategories());
        $this->assign('customers',$customer->listByKey());
        $this->assign('vehicles',$vehicle->listByKey(['id','-','entity_id'],'name'));

        if (!empty($data['entity_id'])) {
            $entity = new Entity($this->app['db']);
            $this->assign('entity', $entity->load($data['entity_id']));
        }
        if (!empty($data['vehicle_id'])) {
            $vehicle = new \BK\Record\Vehicle($this->app['db']);
            $this->assign('vehicle', $vehicle->load($data['vehicle_id']));
        }
    }

    public function _post_editor($id) {
        $this->setTemplate($this->record . '/editor');
        $workorder = new \BK\Record\Workorder($this->app['db']);
        $workorderLine = new WorkorderLine($this->app['db']);
        $invoice = new \BK\Record\Invoice($this->app['db']);
        $transaction = new \BK\Record\Transaction($this->app['db']);
        if (isset($this->params['created_on']) && trim($this->params['created_on']) != '') {
            $this->params['created_on'] = strtotime($this->params['created_on']);
        }
        if (isset($this->params['received_on']) && trim($this->params['received_on']) != '') {
            $this->params['received_on'] = strtotime($this->params['received_on']);
        }
        if (isset($this->params['promised_on']) && trim($this->params['promised_on']) != '') {
            $this->params['promised_on'] = strtotime($this->params['promised_on']);
        }
        if (empty($data['invoice_id'])) {
            $inv = $invoice->getByWorkOrder($id);
            $this->params['invoice_id'] = (!empty($inv) ? $inv['id'] : null);
        }
        $id = $workorder->save($this->params);
        $lines = [];
        foreach ($this->params['deleteLines'] as $lineId) {
            $workorderLine->delete($lineId);
        }
        foreach ($this->params['lines'] as $line) {
            $line['workorder_id'] = $id;
            $line['id'] = $workorderLine->save($line);
            $update = false;
            if (isset($line['cost']) && trim($line['cost']) != '') {
                $cost = [
                    'id' => 'new',
                    'entity_id' => $this->params['entity_id'],
                    'workorder_id' => $id,
                    'workorder_line_id' => $line['id'],
                    'amount' => ((float) $line['cost']) * -1
                ];
                if (trim($line['cost_tran_id']) != '' && (float)$line['cost_tran_id'] > 0) {
                    $cost['id'] = $line['cost_tran_id'];
                }
                $update = true;
                $line['cost_tran_id'] = $transaction->save($cost);
            }
            if ($update) {
                $workorderLine->save($line);
            }
            $lines[] = $line;
        }
        $this->assign('id', $id);
        $this->assign('lines', $lines);
    }

    public function _post_delete() {
        $workorder = new \BK\Record\Workorder($this->app['db']);
        $workorderLine = new WorkorderLine($this->app['db']);
        $transaction = new \BK\Record\Transaction($this->app['db']);
        $invoiceRoute = new Invoice();
        $data = $workorder->load($this->params['id']);
        $data['lines'] = $workorderLine->getByWorkOrder($this->params['id']);
        $trans = $transaction->listAll(' workorder_id = ?',[$this->params['id']]);
        foreach ($data['lines'] as $line) {
            $workorderLine->delete($line['id']);
        }
        foreach ($trans as $tran) {
            $transaction->delete($tran['id']);
        }
        $invoiceRoute->params = ['id' => $data['invoice_id']];
        $invoiceRoute->_post_delete($data['invoice_id']);
        $workorder->delete($this->params['id']);
        $this->setTemplate($this->record . '/editor');
    }


    public function _get_print($id) {
        $this->_get_editor($id);
        $setting = new \BK\Record\Setting($this->app['db']);
        $this->js('jsPDF.2.5.1');
        $this->js('html2canv.1.4.1');
        $this->js('dom-purify.2.3.6.');
        $this->assign('header', $setting->get('business-header'));
        $this->setTemplate($this->record . '/print');
    }
}
